

----------------------< Mike Nelson's text formating routines>-------------------------
global function unary_function(integer id,object Param)
   if atom(Param) then return call_func(id,{Param}) end if
   for i=1 to length(Param) do
      Param[i]=call_func(id,{Param[i]})
   end for
   return Param
end function   

global function binary_function(integer id,object Param1,object Param2)
   integer len1,len2
   if atom(Param1) then len1=-1 else len1=length(Param1) end if  
   if atom(Param2) then len2=-1 else len2=length(Param2) end if
   if len1=-1 and len2=-1 then
      return call_func(id,{Param1,Param2})
   end if
   if len1=-1 then
      Param1=repeat(Param1,len2)
      len1=len2
   elsif len2=-1 then
      Param2=repeat(Param2,len1)
      len2=len1
   end if
   if len1!=len2 then
      puts(2,"\nSequence length mismatch.\n")
      ? 1/0
   end if
   for i=1 to len1 do
      Param1[i]=call_func(id,{Param1[i],Param2[i]})
   end for
   return Param1
end function

-- formtext.e by Mike Nelson
-- text formatting for DOS/Linux 

include graphics.e 

global constant ALIGN_LEFT=1
global constant ALIGN_CENTER=2
global constant ALIGN_RIGHT=3

integer screenColumns
integer leftMargin
integer rightMargin 
integer tabSize 
integer textAlignment 

global procedure init_text_formatter()
   -- initializes screen parameters
   -- call after changing video modes
   sequence vidconf
   integer cols
   vidconf=video_config()
   screenColumns=vidconf[VC_COLUMNS]
   leftMargin=1
   rightMargin=screenColumns
   tabSize=5
   textAlignment=ALIGN_LEFT
end procedure 
init_text_formatter()  -- first initialization is automatic

global function set_margins(integer left,integer right)    
   -- Sets margins, returns 1. Returns 0 on invalid margin(s).
   -- left=0 or right=0 leaves that margin unchanged
   -- a negative value for right indicates moving left 
   if left<0 then return 0 end if
   if right<0 then right=screenColumns+right end if
   if right>screenColumns then return 0 end if
   if left>right then return 0 end if
   if left>0 then leftMargin=left end if
   if right>0 then rightMargin=right end if 
   return 1   
end function                     

global function get_margins()
   return {leftMargin,rightMargin}   
end function

global function set_tabsize(integer size)
   -- sets tabsize, returns 1.  Returns 0 on invalid size.
   if size<0 then return 0 end if
   if size>rightMargin-leftMargin+1 then return 0 end if
   tabSize=size
   return 1
end function 

global function get_tabsize()
   return tabSize  
end function

global function set_alignment(integer align)
   --sets alignment, returns 1. Returns 0 on invalid alignment
   if align<ALIGN_LEFT or align>ALIGN_RIGHT then return 0 end if
   textAlignment=align
   return 1
end function

global function get_alignment()
   return textAlignment   
end function             

function ltrim(sequence text)
   for i=1 to length(text) do
      if text[i]!=' ' then
	 return text[i..length(text)]
      end if
   end for
   return ""
end function 

function rtrim(sequence text)
   for i=length(text) to 1 by -1 do
      if text[i]!=' ' then
	 return text[1..i]   
      end if
   end for
   return ""
end function

procedure print_aligned(sequence text,integer width)
   sequence lspace,rspace
   integer spaces 
   spaces=width-length(text)
   if textAlignment=ALIGN_LEFT then
      lspace=""
      rspace=repeat(' ',spaces)
   elsif textAlignment=ALIGN_CENTER then
      lspace=repeat(' ',floor(spaces/2))
      rspace=repeat(' ',floor((spaces+1)/2))
   else
      lspace=repeat(' ',spaces)
      rspace=""
   end if
   puts(1,lspace&text&rspace)
end procedure

global procedure print_formatted(sequence text)
--Prints formatted text to screen starting at current postion,
--using the curent margin and alignment settings. 
--Output is word-wrapped if possible, hyphenated if not.
--Tabs are expanded to the number of spaces given by the current tabsize
--setting.
--Whitespace is trimmed only at word wraps and hyphenations.
--Assumes text is a valid string.
   integer width,offset,index
   sequence pos,printText
   while 1 do
      index=find('\t',text)
      if index=0 then exit end if
      text=text[1..index-1]&repeat(' ',tabSize)&text[index+1..length(text)]
   end while
   width=rightMargin-leftMargin+1 
   pos=get_position()
   offset=0
   if pos[2]>rightMargin then
      puts(1,"\n")
      pos=get_position()
   end if
   if pos[2]<leftMargin then
      position(pos[1],leftMargin)
   elsif pos[2]>leftMargin then
      offset=pos[2]-leftMargin 
   end if
   index=find('\n',text)
   if index=0 then index=length(text)+1 end if
   if index<=width-offset+1 then
      print_aligned(text[1..index-1],width-offset)
      if index<=length(text) then
	 print_formatted(text[index+1..length(text)])
      end if
      return
   end if
   for i=width-offset+1 to 1 by -1 do
      if text[i]=' ' then
	 print_aligned(rtrim(text[1..i-1]),width)
	 print_formatted(ltrim(text[i+1..length(text)]))
	 return
      end if
   end for
   print_aligned(text[1..width-offset-1]&'-',width-offset)
   if length(text)>=width-offset then
      print_formatted(ltrim(text[width-offset..length(text)]))
   end if
end procedure
-----------------------------------------------------------------------------------------   

global procedure wipe(sequence start_wipe, sequence finish_wipe)
    for line = start_wipe[1]  to finish_wipe[1] do
	position(line, start_wipe[2])
	puts(1, repeat(' ',(finish_wipe[2])))
    end for
end procedure       

 global procedure quit()
      text_color(WHITE)
      puts(1,'\n')
      clear_screen()
	  abort(1)
 end procedure

-----<rounding routines from Ricardo Forno's genfunc.e library in archives>-----
global function RoundCents(object a, object cent)
    integer len
    sequence s
    object t, u
    if atom(a) then
	if atom(cent) then
	    return floor(a * cent + 0.5) / cent
	end if
	len = length(cent)
	s = repeat(0, len)
	for i = 1 to len do
	    t = cent[i]
	    if atom (t) then
		s[i] = floor(a * t + 0.5) / t
	    else
		s[i] = RoundCents(a, t)
	    end if
	end for
	return s
    elsif atom(cent) then
	len = length(a)
	s = repeat(0, len)
	for i = 1 to len do
	    t = a[i]
	    if atom(t) then
		s[i] = floor(t * cent + 0.5) / cent
	    else
		s[i] = RoundCents(t, cent)
	    end if
	end for
	return s
    end if
    len = length(a)
    if len != length(cent) then
	abort(1)
    end if
    s = repeat(0, len)
    for i = 1 to len do
	t = cent[i]
	if atom(t) then
	    u = a[i]
	    if atom(u) then
		s[i] = floor(u * t + 0.5) / t
	    else
		s[i] = RoundCents(u, t)
	    end if
	else
	    s[i] = RoundCents(a[i], t)
	end if
    end for
    return s
end function

global function Round(object a)
    object t
    if integer(a) then
	return a
    elsif atom(a) then
	return floor(a + 0.5)
    end if
    for i = 1 to length(a) do
	t = a[i]
	if integer(t) then
	elsif atom(t) then
	    a[i] = floor(t + 0.5)
	else
	    a[i] = Round(t)
	end if
    end for
    return a
end function

------------------------------------------------------------------------------
include graphics.e
constant VC = video_config()
global procedure CenterDisplay(integer line, integer lm, integer rm, 
				integer color, sequence text)
    integer column
--  column = Round((rm-lm)/2-(length(text)/2))
    column = Round((VC[VC_COLUMNS])/2-(length(text)/2))
    position (line, column)
    text_color(color)
    printf(1, "%s", {text})
end procedure

global function Prompt_For_Nput(sequence pos, integer color, sequence text)
    sequence input_start
    position(pos[1],pos[2])
    text_color(color)
    puts(1, text)
    input_start = get_position()
    return input_start
end function
------------------------------------------------------------------------------
--------------------------------<  Nput() >-----------------------------------
------------------------------------------------------------------------------

global function Nput(sequence label, sequence input_start, sequence range, integer rm)

-- Prompt the user to enter a number. 
-- A range of allowed values may be specified.
    object answer
    atom inpcolor, sign
    sequence buffer -- holds length of input
    integer size
    inpcolor = label[1]
    sign = label[2]
    while 1 do
	 text_color(inpcolor)
	 position(input_start[1], input_start[2])
	 printf(1,"%s", sign)

	 size = rm - input_start[2]
	 puts(1, repeat(' ',size))
	 position(input_start[1],input_start[2]+2)
	 
	 buffer = gets(0) -- make sure whole line is read
	 
	 answer = value(buffer)
	 
	 position(input_start[1],input_start[2])
	 
	 if answer[1] != GET_SUCCESS or sequence(answer[2]) then
		puts(1, repeat( ' ', length(buffer))) 
	 else
	     if length(range) = 2 then
		  if range[1] <= answer[2] and answer[2] <= range[2] then
		      return answer[2]
		  else
		    puts(1, repeat( ' ', length(buffer))) 
		  end if
	      else
		  return answer[2]
	      end if
	 end if
    end while
end function

